"""
    Exemple de module permettant de faire des fonctions de correction
    (recherche du minimum + reherche des occurences d'une valeur dans un 
    tableau)
"""

import inspect
from IPython.display import HTML



""" ***************************************************************************
* Partie 1 : fonctions pour créer un tableau HTML contenant les comparaisons  *
* des fonctions avec leurs fonctions de correction sur un jeu de tests        *
*************************************************************************** """

#Couleurs d'affichage à modifier en cas de besoin
color_head = 'LightBlue'
color_true = 'LightGreen'
color_false = 'LightSalmon'
color= ""

def html_head():
    """ 
        Fonction qui retourne l'entête du tableau HTML contenant les résultats 
        du test.
    """
    code_html_head = """<table style="width:100%; margin:auto;">
                            <thead style='background-color:{c};'>
                                <tr>
                                    <th style='text-align:center;'>Paramètres</th>
                                    <th style='text-align:center;'>Résultat attendu</th>
                                    <th style='text-align:center;'>Résultat obtenu</th>
                                    <th style='text-align:center;'>Validation</th>
                                </tr>
                            </thead>
                            <tbody>""".format(c=color_head)
    return code_html_head

def html_row(fn, arguments):
    """ 
        Fonction qui retourne qui retourne une ligne du tableau HTML contenant 
        un test. La ligne contient le résultat de la comparaison de la fonction
        avec sa fonction de test.
    """
    fn_correction = globals()[fn.__name__+"_correction"] 
    nb_param = len(inspect.signature(fn_correction).parameters)
    # unpacking si le nombre de paramètres est supérieur ou égal à 2    
    if nb_param  > 1 :
        res_fn = fn(*arguments)
        res_fn_correction = fn_correction(*arguments)
    else :
        res_fn = fn(arguments)
        res_fn_correction = fn_correction(arguments)
    # Comparaison des résultats de la fonction fn avec sa fonction de correction
    if res_fn == res_fn_correction :
        valid_color = color_true
        valid_symbol = '&#x2705;'
    else :
        valid_color = color_false
        valid_symbol = '&#x274c;'
    # Construction de la ligne HTML
    code_html_row = """<tr style='background-color:{c};''>
                            <th style='text-align:center;'>{a}</th>
                            <th style='text-align:center;'>{fn_c}</th>
                            <th style='text-align:center;'>{fn}</th>
                            <th style='text-align:center;'>{v}</th>
                        </tr>""".format(c=valid_color, a=arguments, fn_c=res_fn_correction, fn=res_fn, v=valid_symbol)        
    return code_html_row

def html_end_table() :
    """
        Fonction qui retourne la fin du tableau HTML.
    """
    return """</tbody>
        </table>"""

def test_fonction(fn, jeu_test):
    """
        Fonction qui construit le tableau HTML contenant les résultats 
        appliqués à un jeu. Le jeu de tests doit être une liste contenant les
        paramètres pour chaque test.
    """
    code_html = html_head()
    for test in jeu_test :
        code_html += html_row(fn, test)
    code_html += html_end_table()
    return HTML(code_html)

""" ***************************************************************************
* Partie 2 : Fonctions de corrections à placer ici. Elles doivent avoir les   *
* mêmes noms que les fonctions et être suffixées par '_correction' pour       *
* pouvoir être reconnues.                                                     * 
*************************************************************************** """


def minimum_correction(T):
    """
        Fonction de correction pour la recherche du minimum.
    """
    if T == [] :
        return None
    else :
        imin = 0
        for i in range(1, len(T)):
            if T[i] < T[imin] :
                imin = i
        return T[imin]
    
def occurences_correction(T, x):
    """
        Fonction de correction pour la recherche des occurences de x dans le
        tableau T.
    """
    occ = []
    for i in range(len(T)):
        if T[i] == x :
            occ.append(i)
    return occ
