"""
    Exemple de module permettant de faire des fonctions de correction pour les
    algorithmes gloutons :
        - problème du rendu de monnaie
        - problème du sac à dos
        - problème du voyageur de commerce
"""

import inspect
from IPython.display import HTML

""" ***************************************************************************
* Partie 1 : fonctions pour créer un tableau HTML contenant les comparaisons  *
* des fonctions avec leurs fonctions de correction sur un jeu de tests        *
*************************************************************************** """

#Couleurs d'affichage à modifier en cas de besoin
color_head = 'LightBlue'
color_true = 'LightGreen'
color_false = 'LightSalmon'
color= ""

def html_head():
    """ 
        Fonction qui retourne l'entête du tableau HTML contenant les résultats 
        du test.
    """
    code_html_head = """<table style="width:100%; margin:auto;">
                            <thead style='background-color:{c};'>
                                <tr>
                                    <th style='text-align:center;'>Paramètres</th>
                                    <th style='text-align:center;'>Résultat attendu</th>
                                    <th style='text-align:center;'>Résultat obtenu</th>
                                    <th style='text-align:center;'>Validation</th>
                                </tr>
                            </thead>
                            <tbody>""".format(c=color_head)
    return code_html_head

def html_row(fn, arguments):
    """ 
        Fonction qui retourne qui retourne une ligne du tableau HTML contenant 
        un test. La ligne contient le résultat de la comparaison de la fonction
        avec sa fonction de test.
    """
    fn_correction = globals()[fn.__name__+"_correction"] 
    nb_param = len(inspect.signature(fn_correction).parameters)
    # unpacking si le nombre de paramètres est supérieur ou égal à 2    
    if nb_param  > 1 :
        res_fn = fn(*arguments)
        res_fn_correction = fn_correction(*arguments)
    else :
        res_fn = fn(arguments)
        res_fn_correction = fn_correction(arguments)
    # Comparaison des résultats de la fonction fn avec sa fonction de correction
    if res_fn == res_fn_correction :
        valid_color = color_true
        valid_symbol = '&#x2705;'
    else :
        valid_color = color_false
        valid_symbol = '&#x274c;'
    # Construction de la ligne HTML
    code_html_row = """<tr style='background-color:{c};''>
                            <th style='text-align:center;'>{a}</th>
                            <th style='text-align:center;'>{fn_c}</th>
                            <th style='text-align:center;'>{fn}</th>
                            <th style='text-align:center;'>{v}</th>
                        </tr>""".format(c=valid_color, a=arguments, fn_c=res_fn_correction, fn=res_fn, v=valid_symbol)        
    return code_html_row

def html_end_table() :
    """
        Fonction qui retourne la fin du tableau HTML.
    """
    return """</tbody>
        </table>"""

def test_fonction(fn, jeu_test):
    """
        Fonction qui construit le tableau HTML contenant les résultats 
        appliqués à un jeu. Le jeu de tests doit être une liste contenant les
        paramètres pour chaque test.
    """
    code_html = html_head()
    for test in jeu_test :
        code_html += html_row(fn, test)
    code_html += html_end_table()
    return HTML(code_html)

""" ***************************************************************************
* Partie 2 : Fonctions de corrections à placer ici. Elles doivent avoir les   *
* mêmes noms que les fonctions et être suffixées par '_correction' pour       *
* pouvoir être reconnues.                                                     * 
*************************************************************************** """

def solution_glouton_monnaie_correction(somme, pieces):
    """
        Fonction qui retourne le nombre de pièces rendues pour une somme
        donnée et pour un système de pièces donné avec un choix glouton.
    """
    N = len(pieces)
    rendu = N*[0]
    k = N-1
    while somme > 0 :
        if pieces[k] <= somme :
            somme = somme - pieces[k]
            rendu[k] = rendu[k] + 1
        else :
            k = k - 1
    return rendu

def solution_glouton_sac_correction(masse, liste_obj):
    """
        Fonction qui retourne une liste d'objets rentrant dans un sac de masse
        donnée pour une liste d'objets disponibles en appliquant un choix
        glouton. La listes d'objets est parcourue de la fin vers le début. 
        Elle doit donc être triée suivant le bon critère. 
    """
    N = len(liste_obj)
    sac = []
    k = N-1
    while k >= 0 :
        if liste_obj[k][2] <= masse :
            masse = masse - liste_obj[k][2]
            sac.append(liste_obj[k])
        k = k - 1
    return sac

import math

def solution_glouton_voyageur_correction(dep, distances):
    """
        Fonction qui retourne la solution gloutonne du problème du voyageur de
        commerce pour une ville de départ donnée et un dictionnaire contenant
        les distances entre les différentes villes. Le résultat retourné est
        un tuple contenant le trajet et la distance totale parcourue.
    """
    d_total = 0
    ville = dep
    trajet = [dep]
    N = len(distances) - 1
    k = 0
    while k < N :
        # 1) Recherche de la ville la plus proche non parcourue en partant de 
        # la ville courante
        v_min = None
        d_min = math.inf
        for v, d in distances[ville].items() :
            if not v in trajet and d < d_min :
                v_min = v
                d_min = d
        # 2) Ajout de la ville au trajet
        d_total = d_total + d_min
        trajet.append(v_min)
        # 3) Passage à la ville suivante
        ville = v_min
        k = k + 1
    # 4) Retour à la ville de départ
    trajet.append(dep)
    d_total = d_total + distances[ville][dep]
    return trajet, d_total